#include "gamedata/MappingToGraphic.h"
#include "exception/OutOfRangeIndexException.h"

namespace Tales {


MappingToGraphicEntry MappingToGraphic::mappingToGraphic(int mappingIndex) {
  if ((unsigned int)(mappingIndex) >= numMappings_) {
    throw OutOfRangeIndexException(TALES_SRCANDLINE,
                                   "MappingToGraphic::mappingToGraphic(int)",
                                   mappingIndex);
  }
  
  return mappingToGraphic_[mappingIndex];
}

const MappingToGraphicEntry MappingToGraphic::mappingToGraphic_[numMappings_] =
  {
  // obj 0x01 (tails)
  MappingToGraphicEntry(0x000, GraphicIdentifier::uncompressed, 0x000, 0x00),
  MappingToGraphicEntry(0x001, GraphicIdentifier::uncompressed, 0x001, 0x00),
  MappingToGraphicEntry(0x002, GraphicIdentifier::uncompressed, 0x002, 0x00),
  MappingToGraphicEntry(0x003, GraphicIdentifier::uncompressed, 0x003, 0x00),
  MappingToGraphicEntry(0x004, GraphicIdentifier::uncompressed, 0x004, 0x00),
  MappingToGraphicEntry(0x005, GraphicIdentifier::uncompressed, 0x005, 0x00),
  MappingToGraphicEntry(0x006, GraphicIdentifier::uncompressed, 0x006, 0x00),
  MappingToGraphicEntry(0x007, GraphicIdentifier::uncompressed, 0x007, 0x00),
  MappingToGraphicEntry(0x008, GraphicIdentifier::uncompressed, 0x008, 0x00),
  MappingToGraphicEntry(0x009, GraphicIdentifier::uncompressed, 0x009, 0x00),
  MappingToGraphicEntry(0x00A, GraphicIdentifier::uncompressed, 0x00A, 0x00),
  MappingToGraphicEntry(0x00B, GraphicIdentifier::uncompressed, 0x00B, 0x00),
  MappingToGraphicEntry(0x00C, GraphicIdentifier::uncompressed, 0x00C, 0x00),
  MappingToGraphicEntry(0x00D, GraphicIdentifier::uncompressed, 0x00D, 0x00),
  MappingToGraphicEntry(0x00E, GraphicIdentifier::uncompressed, 0x00E, 0x00),
  MappingToGraphicEntry(0x00F, GraphicIdentifier::uncompressed, 0x00F, 0x00),
  MappingToGraphicEntry(0x010, GraphicIdentifier::uncompressed, 0x010, 0x00),
  MappingToGraphicEntry(0x011, GraphicIdentifier::uncompressed, 0x011, 0x00),
  MappingToGraphicEntry(0x012, GraphicIdentifier::uncompressed, 0x012, 0x00),
  MappingToGraphicEntry(0x013, GraphicIdentifier::uncompressed, 0x013, 0x00),
  MappingToGraphicEntry(0x014, GraphicIdentifier::uncompressed, 0x014, 0x00),
  MappingToGraphicEntry(0x015, GraphicIdentifier::uncompressed, 0x015, 0x00),
  MappingToGraphicEntry(0x016, GraphicIdentifier::uncompressed, 0x016, 0x00),
  MappingToGraphicEntry(0x017, GraphicIdentifier::uncompressed, 0x017, 0x00),
  MappingToGraphicEntry(0x018, GraphicIdentifier::uncompressed, 0x018, 0x00),
  MappingToGraphicEntry(0x019, GraphicIdentifier::uncompressed, 0x019, 0x00),
  MappingToGraphicEntry(0x01A, GraphicIdentifier::uncompressed, 0x01A, 0x00),
  MappingToGraphicEntry(0x01B, GraphicIdentifier::uncompressed, 0x01B, 0x00),
  MappingToGraphicEntry(0x01C, GraphicIdentifier::uncompressed, 0x01C, 0x00),
  MappingToGraphicEntry(0x01D, GraphicIdentifier::uncompressed, 0x01D, 0x00),
  MappingToGraphicEntry(0x01E, GraphicIdentifier::uncompressed, 0x01E, 0x00),
  MappingToGraphicEntry(0x01F, GraphicIdentifier::uncompressed, 0x01F, 0x00),
  MappingToGraphicEntry(0x020, GraphicIdentifier::uncompressed, 0x020, 0x00),
  MappingToGraphicEntry(0x021, GraphicIdentifier::uncompressed, 0x021, 0x00),
  MappingToGraphicEntry(0x022, GraphicIdentifier::uncompressed, 0x022, 0x00),
  MappingToGraphicEntry(0x023, GraphicIdentifier::uncompressed, 0x023, 0x00),
  MappingToGraphicEntry(0x024, GraphicIdentifier::uncompressed, 0x024, 0x00),
  MappingToGraphicEntry(0x025, GraphicIdentifier::uncompressed, 0x025, 0x00),
  MappingToGraphicEntry(0x026, GraphicIdentifier::uncompressed, 0x026, 0x00),
  MappingToGraphicEntry(0x027, GraphicIdentifier::uncompressed, 0x027, 0x00),
  MappingToGraphicEntry(0x028, GraphicIdentifier::uncompressed, 0x028, 0x00),
  MappingToGraphicEntry(0x029, GraphicIdentifier::uncompressed, 0x029, 0x00),
  MappingToGraphicEntry(0x02A, GraphicIdentifier::uncompressed, 0x02A, 0x00),
  MappingToGraphicEntry(0x02B, GraphicIdentifier::uncompressed, 0x02B, 0x00),
  MappingToGraphicEntry(0x02C, GraphicIdentifier::uncompressed, 0x02C, 0x00),
  MappingToGraphicEntry(0x02D, GraphicIdentifier::uncompressed, 0x02D, 0x00),
  MappingToGraphicEntry(0x02E, GraphicIdentifier::uncompressed, 0x02E, 0x00),
  MappingToGraphicEntry(0x02F, GraphicIdentifier::uncompressed, 0x02F, 0x00),
  MappingToGraphicEntry(0x030, GraphicIdentifier::uncompressed, 0x030, 0x00),
  MappingToGraphicEntry(0x031, GraphicIdentifier::uncompressed, 0x031, 0x00),
  MappingToGraphicEntry(0x032, GraphicIdentifier::uncompressed, 0x032, 0x00),
  MappingToGraphicEntry(0x033, GraphicIdentifier::uncompressed, 0x033, 0x00),
  MappingToGraphicEntry(0x034, GraphicIdentifier::uncompressed, 0x034, 0x00),
  MappingToGraphicEntry(0x035, GraphicIdentifier::uncompressed, 0x035, 0x00),
  MappingToGraphicEntry(0x036, GraphicIdentifier::uncompressed, 0x036, 0x00),
  MappingToGraphicEntry(0x037, GraphicIdentifier::uncompressed, 0x037, 0x00),
  MappingToGraphicEntry(0x038, GraphicIdentifier::uncompressed, 0x038, 0x00),
  MappingToGraphicEntry(0x039, GraphicIdentifier::uncompressed, 0x039, 0x00),
  MappingToGraphicEntry(0x03A, GraphicIdentifier::uncompressed, 0x03A, 0x00),
  MappingToGraphicEntry(0x03B, GraphicIdentifier::uncompressed, 0x03B, 0x00),
  MappingToGraphicEntry(0x03C, GraphicIdentifier::uncompressed, 0x03C, 0x00),
  MappingToGraphicEntry(0x03D, GraphicIdentifier::uncompressed, 0x03D, 0x00),
  MappingToGraphicEntry(0x03E, GraphicIdentifier::uncompressed, 0x03E, 0x00),
  MappingToGraphicEntry(0x03F, GraphicIdentifier::uncompressed, 0x03F, 0x00),
  MappingToGraphicEntry(0x040, GraphicIdentifier::uncompressed, 0x040, 0x00),
  MappingToGraphicEntry(0x041, GraphicIdentifier::uncompressed, 0x041, 0x00),
  MappingToGraphicEntry(0x042, GraphicIdentifier::uncompressed, 0x042, 0x00),
  MappingToGraphicEntry(0x043, GraphicIdentifier::uncompressed, 0x043, 0x00),
  MappingToGraphicEntry(0x044, GraphicIdentifier::uncompressed, 0x044, 0x00),
  MappingToGraphicEntry(0x045, GraphicIdentifier::uncompressed, 0x045, 0x00),
  MappingToGraphicEntry(0x046, GraphicIdentifier::uncompressed, 0x046, 0x00),
  MappingToGraphicEntry(0x047, GraphicIdentifier::uncompressed, 0x047, 0x00),
  MappingToGraphicEntry(0x048, GraphicIdentifier::uncompressed, 0x048, 0x00),
  MappingToGraphicEntry(0x049, GraphicIdentifier::uncompressed, 0x049, 0x00),
  MappingToGraphicEntry(0x04A, GraphicIdentifier::uncompressed, 0x04A, 0x00),
  MappingToGraphicEntry(0x04B, GraphicIdentifier::uncompressed, 0x04B, 0x00),
  MappingToGraphicEntry(0x04C, GraphicIdentifier::uncompressed, 0x04C, 0x00),
  MappingToGraphicEntry(0x04D, GraphicIdentifier::uncompressed, 0x04D, 0x00),
  MappingToGraphicEntry(0x04E, GraphicIdentifier::uncompressed, 0x04E, 0x00),
  MappingToGraphicEntry(0x04F, GraphicIdentifier::uncompressed, 0x04F, 0x00),
  MappingToGraphicEntry(0x050, GraphicIdentifier::uncompressed, 0x050, 0x00),
  MappingToGraphicEntry(0x051, GraphicIdentifier::uncompressed, 0x051, 0x00),
  MappingToGraphicEntry(0x052, GraphicIdentifier::uncompressed, 0x052, 0x00),
  MappingToGraphicEntry(0x053, GraphicIdentifier::uncompressed, 0x053, 0x00),
  MappingToGraphicEntry(0x054, GraphicIdentifier::uncompressed, 0x054, 0x00),
  MappingToGraphicEntry(0x055, GraphicIdentifier::uncompressed, 0x055, 0x00),
  MappingToGraphicEntry(0x056, GraphicIdentifier::uncompressed, 0x056, 0x00),
  MappingToGraphicEntry(0x057, GraphicIdentifier::uncompressed, 0x057, 0x00),
  MappingToGraphicEntry(0x058, GraphicIdentifier::uncompressed, 0x058, 0x00),
  MappingToGraphicEntry(0x059, GraphicIdentifier::uncompressed, 0x059, 0x00),
  MappingToGraphicEntry(0x05A, GraphicIdentifier::uncompressed, 0x05A, 0x00),
  MappingToGraphicEntry(0x05B, GraphicIdentifier::uncompressed, 0x05B, 0x00),
  MappingToGraphicEntry(0x05C, GraphicIdentifier::uncompressed, 0x05C, 0x00),
  MappingToGraphicEntry(0x05D, GraphicIdentifier::uncompressed, 0x05D, 0x00),
  MappingToGraphicEntry(0x05E, GraphicIdentifier::uncompressed, 0x05E, 0x00),
  MappingToGraphicEntry(0x05F, GraphicIdentifier::uncompressed, 0x05F, 0x00),
  MappingToGraphicEntry(0x060, GraphicIdentifier::uncompressed, 0x060, 0x00),
  MappingToGraphicEntry(0x061, GraphicIdentifier::uncompressed, 0x061, 0x00),
  MappingToGraphicEntry(0x062, GraphicIdentifier::uncompressed, 0x062, 0x00),
  MappingToGraphicEntry(0x063, GraphicIdentifier::uncompressed, 0x063, 0x00),
  MappingToGraphicEntry(0x064, GraphicIdentifier::uncompressed, 0x064, 0x00),
  MappingToGraphicEntry(0x065, GraphicIdentifier::uncompressed, 0x065, 0x00),
  MappingToGraphicEntry(0x066, GraphicIdentifier::uncompressed, 0x066, 0x00),
  MappingToGraphicEntry(0x067, GraphicIdentifier::uncompressed, 0x067, 0x00),
  MappingToGraphicEntry(0x068, GraphicIdentifier::uncompressed, 0x068, 0x00),
  MappingToGraphicEntry(0x069, GraphicIdentifier::uncompressed, 0x069, 0x00),
  MappingToGraphicEntry(0x06A, GraphicIdentifier::uncompressed, 0x06A, 0x00),
  MappingToGraphicEntry(0x06B, GraphicIdentifier::uncompressed, 0x06B, 0x00),
  MappingToGraphicEntry(0x06C, GraphicIdentifier::uncompressed, 0x06C, 0x00),
  MappingToGraphicEntry(0x06D, GraphicIdentifier::uncompressed, 0x06D, 0x00),
  MappingToGraphicEntry(0x06E, GraphicIdentifier::uncompressed, 0x06E, 0x00),
  MappingToGraphicEntry(0x06F, GraphicIdentifier::uncompressed, 0x06F, 0x00),
  MappingToGraphicEntry(0x070, GraphicIdentifier::uncompressed, 0x070, 0x00),
  MappingToGraphicEntry(0x071, GraphicIdentifier::uncompressed, 0x071, 0x00),
  MappingToGraphicEntry(0x072, GraphicIdentifier::uncompressed, 0x072, 0x00),
  MappingToGraphicEntry(0x073, GraphicIdentifier::uncompressed, 0x073, 0x00),
  MappingToGraphicEntry(0x074, GraphicIdentifier::uncompressed, 0x074, 0x00),
  MappingToGraphicEntry(0x075, GraphicIdentifier::uncompressed, 0x075, 0x00),
  MappingToGraphicEntry(0x076, GraphicIdentifier::uncompressed, 0x076, 0x00),
  MappingToGraphicEntry(0x077, GraphicIdentifier::uncompressed, 0x077, 0x00),
  MappingToGraphicEntry(0x078, GraphicIdentifier::uncompressed, 0x078, 0x00),
  MappingToGraphicEntry(0x079, GraphicIdentifier::uncompressed, 0x079, 0x00),
  MappingToGraphicEntry(0x07A, GraphicIdentifier::uncompressed, 0x07A, 0x00),
  MappingToGraphicEntry(0x07B, GraphicIdentifier::uncompressed, 0x07B, 0x00),
  MappingToGraphicEntry(0x07C, GraphicIdentifier::uncompressed, 0x07C, 0x00),
  MappingToGraphicEntry(0x07D, GraphicIdentifier::uncompressed, 0x07D, 0x00),
  MappingToGraphicEntry(0x07E, GraphicIdentifier::uncompressed, 0x07E, 0x00),
  MappingToGraphicEntry(0x07F, GraphicIdentifier::uncompressed, 0x07F, 0x00),
  MappingToGraphicEntry(0x080, GraphicIdentifier::uncompressed, 0x080, 0x00),
  MappingToGraphicEntry(0x081, GraphicIdentifier::uncompressed, 0x081, 0x00),
  MappingToGraphicEntry(0x082, GraphicIdentifier::uncompressed, 0x082, 0x00),
  MappingToGraphicEntry(0x083, GraphicIdentifier::uncompressed, 0x083, 0x00),
  // 0x02 (remote robot)
  MappingToGraphicEntry(0x084, GraphicIdentifier::uncompressed, 0x084, -0x0A),
  MappingToGraphicEntry(0x085, GraphicIdentifier::uncompressed, 0x085, -0x0A),
  MappingToGraphicEntry(0x086, GraphicIdentifier::uncompressed, 0x086, -0x0A),
  MappingToGraphicEntry(0x087, GraphicIdentifier::uncompressed, 0x087, -0x0A),
  MappingToGraphicEntry(0x088, GraphicIdentifier::uncompressed, 0x088, -0x0A),
  MappingToGraphicEntry(0x089, GraphicIdentifier::uncompressed, 0x089, -0x0A),
  MappingToGraphicEntry(0x08A, GraphicIdentifier::uncompressed, 0x08A, -0x0A),
  MappingToGraphicEntry(0x08B, GraphicIdentifier::uncompressed, 0x08B, -0x0A),
  MappingToGraphicEntry(0x08C, GraphicIdentifier::uncompressed, 0x08C, -0x0A),
  MappingToGraphicEntry(0x08D, GraphicIdentifier::uncompressed, 0x08D, -0x0A),
  MappingToGraphicEntry(0x08E, GraphicIdentifier::uncompressed, 0x08E, -0x0A),
  MappingToGraphicEntry(0x08F, GraphicIdentifier::uncompressed, 0x08F, -0x0A),
  MappingToGraphicEntry(0x090, GraphicIdentifier::uncompressed, 0x090, -0x0A),
  MappingToGraphicEntry(0x091, GraphicIdentifier::uncompressed, 0x091, -0x0A),
  MappingToGraphicEntry(0x092, GraphicIdentifier::uncompressed, 0x092, -0x0A),
  MappingToGraphicEntry(0x093, GraphicIdentifier::uncompressed, 0x093, -0x0A),
  MappingToGraphicEntry(0x094, GraphicIdentifier::uncompressed, 0x094, -0x0A),
  MappingToGraphicEntry(0x095, GraphicIdentifier::uncompressed, 0x095, -0x0A),
  MappingToGraphicEntry(0x096, GraphicIdentifier::uncompressed, 0x096, -0x0A),
  MappingToGraphicEntry(0x097, GraphicIdentifier::uncompressed, 0x097, -0x0A),
  MappingToGraphicEntry(0x098, GraphicIdentifier::uncompressed, 0x098, -0x0A),
  MappingToGraphicEntry(0x099, GraphicIdentifier::uncompressed, 0x099, -0x0A),
  // 0x03 (extra armor stars)
  MappingToGraphicEntry(0x09A, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x09B, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x09C, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x09D, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x09E, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x09F, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0A0, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0A1, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0A2, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0A3, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0A4, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0A5, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0A6, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0A7, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0A8, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0A9, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0AA, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0AB, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0AC, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0AD, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0AE, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0AF, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0B0, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0B1, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0B2, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0B3, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0B4, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0B5, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0B6, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0B7, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0B8, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  MappingToGraphicEntry(0x0B9, GraphicIdentifier::uncompressed, 0x0DB, -0x4A),
  // 0x04 (item pickup label -- needs special handling)
  MappingToGraphicEntry(0x0BA, GraphicIdentifier::uncompressed, 0x113, -0x5E),
  // 0x05 (ring)
  MappingToGraphicEntry(0x0BB, GraphicIdentifier::compressed, 0x000, -0x12),
  MappingToGraphicEntry(0x0BC, GraphicIdentifier::compressed, 0x000, -0x12),
  MappingToGraphicEntry(0x0BD, GraphicIdentifier::compressed, 0x000, -0x12),
  MappingToGraphicEntry(0x0BE, GraphicIdentifier::compressed, 0x000, -0x12),
  // 0x06, 0x07 (debris -- needs special handling)
  MappingToGraphicEntry(0x0BF, GraphicIdentifier::compressed, 0x03B, -0x5A),
  // 0x09 (water splash)
  MappingToGraphicEntry(0x0C0, GraphicIdentifier::compressed, 0x01A, 0),
  MappingToGraphicEntry(0x0C1, GraphicIdentifier::compressed, 0x01A, 0),
  MappingToGraphicEntry(0x0C2, GraphicIdentifier::compressed, 0x01A, 0),
  MappingToGraphicEntry(0x0C3, GraphicIdentifier::compressed, 0x01A, 0),
  // 0x0A (water ripple effect)
  MappingToGraphicEntry(0x0C4, GraphicIdentifier::compressed, 0x024, 0),
  MappingToGraphicEntry(0x0C5, GraphicIdentifier::compressed, 0x024, 0),
  // 0x0B (cutscene automation object)
  MappingToGraphicEntry(0x0C6, GraphicIdentifier::compressed, 0x02D, 0),
  MappingToGraphicEntry(0x0C7, GraphicIdentifier::compressed, 0x02D, 0),
  MappingToGraphicEntry(0x0C8, GraphicIdentifier::compressed, 0x02D, 0),
  MappingToGraphicEntry(0x0C9, GraphicIdentifier::compressed, 0x02D, 0),
  MappingToGraphicEntry(0x0CA, GraphicIdentifier::compressed, 0x02E, -0x10),
  MappingToGraphicEntry(0x0CB, GraphicIdentifier::compressed, 0x02E, -0x10),
  MappingToGraphicEntry(0x0CC, GraphicIdentifier::compressed, 0x02E, -0x10),
  MappingToGraphicEntry(0x0CD, GraphicIdentifier::compressed, 0x02E, -0x10),
  MappingToGraphicEntry(0x0CE, GraphicIdentifier::compressed, 0x02E, -0x10),
  MappingToGraphicEntry(0x0CF, GraphicIdentifier::compressed, 0x02E, -0x10),
  MappingToGraphicEntry(0x0D0, GraphicIdentifier::compressed, 0x02E, -0x10),
  MappingToGraphicEntry(0x0D1, GraphicIdentifier::compressed, 0x02E, -0x10),
  // 0x0C (Tails' lab claw)
  MappingToGraphicEntry(0x0D2, GraphicIdentifier::compressed, 0x037, 0),
  MappingToGraphicEntry(0x0D3, GraphicIdentifier::compressed, 0x037, 0),
  // 0x0D, 0x0E (explosion)
  MappingToGraphicEntry(0x0D4, GraphicIdentifier::compressed, 0x000, -0x12),
  MappingToGraphicEntry(0x0D5, GraphicIdentifier::compressed, 0x000, -0x12),
  MappingToGraphicEntry(0x0D6, GraphicIdentifier::compressed, 0x000, -0x12),
  MappingToGraphicEntry(0x0D7, GraphicIdentifier::compressed, 0x000, -0x12),
  // 0x10-0x13 (bombs)
  MappingToGraphicEntry(0x0D8, GraphicIdentifier::uncompressed, 0x0C8, 0),
  MappingToGraphicEntry(0x0D9, GraphicIdentifier::uncompressed, 0x0C8, 0),
  // 0x14 (napalm bomb)
  MappingToGraphicEntry(0x0DA, GraphicIdentifier::uncompressed, 0x0C9, 0),
  MappingToGraphicEntry(0x0DB, GraphicIdentifier::uncompressed, 0x0C9, 0),
  MappingToGraphicEntry(0x0DC, GraphicIdentifier::uncompressed, 0x0C9, 0),
  MappingToGraphicEntry(0x0DD, GraphicIdentifier::uncompressed, 0x0C9, 0),
  MappingToGraphicEntry(0x0DE, GraphicIdentifier::uncompressed, 0x0C9, 0),
  // 0x15-0x1A (triple bomb and hammer???)
  MappingToGraphicEntry(0x0DF, GraphicIdentifier::uncompressed, 0x0CB, 0),
  MappingToGraphicEntry(0x0E0, GraphicIdentifier::uncompressed, 0x0CB, 0),
  MappingToGraphicEntry(0x0E1, GraphicIdentifier::uncompressed, 0x0CB, 0),
  // bad???
  MappingToGraphicEntry(0x0E2, GraphicIdentifier::uncompressed, 0x0C8, -0x0C),
  MappingToGraphicEntry(0x0E3, GraphicIdentifier::uncompressed, 0x0C8, -0x0C),
  MappingToGraphicEntry(0x0E4, GraphicIdentifier::uncompressed, 0x0C8, -0x14),
  // 0x1B-0x22 (proton torpedo)
  MappingToGraphicEntry(0x0E5, GraphicIdentifier::uncompressed, 0x09A, 0),
  MappingToGraphicEntry(0x0E6, GraphicIdentifier::uncompressed, 0x09A, 0),
  // 0x23 (vulcan cannon)
  MappingToGraphicEntry(0x0E7, GraphicIdentifier::uncompressed, 0x09C, 0),
  MappingToGraphicEntry(0x0E8, GraphicIdentifier::uncompressed, 0x09C, 0),
  // 0x24-0x26 (anti-air missile)
  MappingToGraphicEntry(0x0E9, GraphicIdentifier::uncompressed, 0x0CA, 0),
  MappingToGraphicEntry(0x0EA, GraphicIdentifier::uncompressed, 0x0CA, 0),
  MappingToGraphicEntry(0x0EB, GraphicIdentifier::uncompressed, 0x0CA, 0),
  // 0x27-0x28 (mine weapon)
  MappingToGraphicEntry(0x0EC, GraphicIdentifier::uncompressed, 0x09B, 0),
  // 0x29-0x2F (item pickup)
  MappingToGraphicEntry(0x0ED, GraphicIdentifier::compressed, 0x038, 0),
  MappingToGraphicEntry(0x0EE, GraphicIdentifier::compressed, 0x038, -0xFC),
  // 0x30 (flying mech duck)
  MappingToGraphicEntry(0x0EF, GraphicIdentifier::compressed, 0x003, 0),
  MappingToGraphicEntry(0x0F0, GraphicIdentifier::compressed, 0x003, 0),
  MappingToGraphicEntry(0x0F1, GraphicIdentifier::compressed, 0x003, 0),
  MappingToGraphicEntry(0x0F2, GraphicIdentifier::compressed, 0x003, 0),
  MappingToGraphicEntry(0x0F3, GraphicIdentifier::compressed, 0x003, 0),
  MappingToGraphicEntry(0x0F4, GraphicIdentifier::compressed, 0x003, 0),
  MappingToGraphicEntry(0x0F5, GraphicIdentifier::compressed, 0x003, 0),
  // 0x31 (walking mech duck)
  MappingToGraphicEntry(0x0F6, GraphicIdentifier::compressed, 0x004, 0),
  MappingToGraphicEntry(0x0F7, GraphicIdentifier::compressed, 0x004, 0),
  MappingToGraphicEntry(0x0F8, GraphicIdentifier::compressed, 0x004, 0),
  // 0x32 (dead duck)
  MappingToGraphicEntry(0x0F9, GraphicIdentifier::compressed, 0x025, 0),
  // 0x33 (bomber mouse)
  MappingToGraphicEntry(0x0FA, GraphicIdentifier::compressed, 0x005, 0),
  MappingToGraphicEntry(0x0FB, GraphicIdentifier::compressed, 0x005, 0),
  MappingToGraphicEntry(0x0FC, GraphicIdentifier::compressed, 0x005, 0),
  MappingToGraphicEntry(0x0FD, GraphicIdentifier::compressed, 0x005, 0),
  // 0x34-0x35 (fixed-position gun)
  MappingToGraphicEntry(0x0FE, GraphicIdentifier::compressed, 0x006, 0),
  MappingToGraphicEntry(0x0FF, GraphicIdentifier::compressed, 0x006, 0),
  MappingToGraphicEntry(0x100, GraphicIdentifier::compressed, 0x006, 0),
  // 0x36 (crouching duck)
  MappingToGraphicEntry(0x101, GraphicIdentifier::compressed, 0x007, 0),
  MappingToGraphicEntry(0x102, GraphicIdentifier::compressed, 0x007, 0),
  MappingToGraphicEntry(0x103, GraphicIdentifier::compressed, 0x007, 0),
  // 0x37 (drill robot)
  MappingToGraphicEntry(0x104, GraphicIdentifier::compressed, 0x008, 0),
  MappingToGraphicEntry(0x105, GraphicIdentifier::compressed, 0x008, 0),
  // 0x38 (flying bat)
  MappingToGraphicEntry(0x106, GraphicIdentifier::compressed, 0x00B, 0),
  MappingToGraphicEntry(0x107, GraphicIdentifier::compressed, 0x00B, 0),
  MappingToGraphicEntry(0x108, GraphicIdentifier::compressed, 0x00B, 0),
  MappingToGraphicEntry(0x109, GraphicIdentifier::compressed, 0x00B, 0),
  // 0x39 (bomb and flee duck)
  MappingToGraphicEntry(0x10A, GraphicIdentifier::compressed, 0x00C, 0),
  MappingToGraphicEntry(0x10B, GraphicIdentifier::compressed, 0x00C, 0),
  MappingToGraphicEntry(0x10C, GraphicIdentifier::compressed, 0x00C, 0),
  MappingToGraphicEntry(0x10D, GraphicIdentifier::compressed, 0x00C, 0),
  MappingToGraphicEntry(0x10E, GraphicIdentifier::compressed, 0x00C, 0),
  MappingToGraphicEntry(0x10F, GraphicIdentifier::compressed, 0x00C, 0),
  // 0x3A (jumping lava)
  MappingToGraphicEntry(0x110, GraphicIdentifier::compressed, 0x01F, 0),
  MappingToGraphicEntry(0x111, GraphicIdentifier::compressed, 0x01F, 0),
  MappingToGraphicEntry(0x112, GraphicIdentifier::compressed, 0x01F, 0),
  MappingToGraphicEntry(0x113, GraphicIdentifier::compressed, 0x01F, 0),
  MappingToGraphicEntry(0x114, GraphicIdentifier::compressed, 0x01F, 0),
  MappingToGraphicEntry(0x115, GraphicIdentifier::compressed, 0x01F, 0),
  MappingToGraphicEntry(0x116, GraphicIdentifier::compressed, 0x01F, 0),
  MappingToGraphicEntry(0x117, GraphicIdentifier::compressed, 0x01F, 0),
  MappingToGraphicEntry(0x118, GraphicIdentifier::compressed, 0x01F, 0),
  MappingToGraphicEntry(0x119, GraphicIdentifier::compressed, 0x01F, 0),
  // 0x3B (gunner duck)
  MappingToGraphicEntry(0x11A, GraphicIdentifier::compressed, 0x015, 0),
  MappingToGraphicEntry(0x11B, GraphicIdentifier::compressed, 0x015, 0),
  MappingToGraphicEntry(0x11C, GraphicIdentifier::compressed, 0x015, 0),
  MappingToGraphicEntry(0x11D, GraphicIdentifier::compressed, 0x015, 0),
  // 0x3C (leaping duck)
  MappingToGraphicEntry(0x11E, GraphicIdentifier::compressed, 0x00D, 0),
  MappingToGraphicEntry(0x11F, GraphicIdentifier::compressed, 0x00D, 0),
  MappingToGraphicEntry(0x120, GraphicIdentifier::compressed, 0x00D, 0),
  // 0x3D (shoot and flee duck)
  MappingToGraphicEntry(0x121, GraphicIdentifier::compressed, 0x00E, 0),
  MappingToGraphicEntry(0x122, GraphicIdentifier::compressed, 0x00E, 0),
  MappingToGraphicEntry(0x123, GraphicIdentifier::compressed, 0x00E, 0),
  // 0x3E (rock-throwing duck)
  MappingToGraphicEntry(0x124, GraphicIdentifier::compressed, 0x00F, 0),
  MappingToGraphicEntry(0x125, GraphicIdentifier::compressed, 0x00F, 0),
  MappingToGraphicEntry(0x126, GraphicIdentifier::compressed, 0x00F, 0),
  MappingToGraphicEntry(0x127, GraphicIdentifier::compressed, 0x00F, 0),
  // 0x3F (fighter duck)
  MappingToGraphicEntry(0x128, GraphicIdentifier::compressed, 0x027, 0),
  MappingToGraphicEntry(0x129, GraphicIdentifier::compressed, 0x027, 0),
  MappingToGraphicEntry(0x12A, GraphicIdentifier::compressed, 0x027, 0),
  MappingToGraphicEntry(0x12B, GraphicIdentifier::compressed, 0x027, 0),
  MappingToGraphicEntry(0x12C, GraphicIdentifier::compressed, 0x027, 0),
  MappingToGraphicEntry(0x12D, GraphicIdentifier::compressed, 0x027, 0),
  MappingToGraphicEntry(0x12E, GraphicIdentifier::compressed, 0x027, 0),
  MappingToGraphicEntry(0x12F, GraphicIdentifier::compressed, 0x027, 0),
  MappingToGraphicEntry(0x130, GraphicIdentifier::compressed, 0x027, 0),
  MappingToGraphicEntry(0x131, GraphicIdentifier::compressed, 0x027, 0),
  MappingToGraphicEntry(0x132, GraphicIdentifier::compressed, 0x027, 0),
  // 0x40 (fly and bomb duck)
  MappingToGraphicEntry(0x133, GraphicIdentifier::compressed, 0x010, 0),
  MappingToGraphicEntry(0x134, GraphicIdentifier::compressed, 0x010, 0),
  MappingToGraphicEntry(0x135, GraphicIdentifier::compressed, 0x010, 0),
  MappingToGraphicEntry(0x136, GraphicIdentifier::compressed, 0x010, 0),
  MappingToGraphicEntry(0x137, GraphicIdentifier::compressed, 0x010, 0),
  // 0x41 (mine hazard)
  MappingToGraphicEntry(0x138, GraphicIdentifier::compressed, 0x011, 0),
  // 0x42 (underwater missile hazard)
  MappingToGraphicEntry(0x139, GraphicIdentifier::compressed, 0x012, 0),
  // 0x43 (mini duck)
  MappingToGraphicEntry(0x13A, GraphicIdentifier::compressed, 0x013, 0),
  MappingToGraphicEntry(0x13B, GraphicIdentifier::compressed, 0x013, 0),
  MappingToGraphicEntry(0x13C, GraphicIdentifier::compressed, 0x013, 0),
  MappingToGraphicEntry(0x13D, GraphicIdentifier::compressed, 0x013, 0),
  MappingToGraphicEntry(0x13E, GraphicIdentifier::compressed, 0x013, 0),
  MappingToGraphicEntry(0x13F, GraphicIdentifier::compressed, 0x013, 0),
  // 0x44 (conveyor bomb spawner)
  MappingToGraphicEntry(0x140, GraphicIdentifier::compressed, 0x018, 0),
  MappingToGraphicEntry(0x141, GraphicIdentifier::compressed, 0x018, 0),
  // 0x45 (robot bird launcher)
  MappingToGraphicEntry(0x142, GraphicIdentifier::compressed, 0x014, 0),
  MappingToGraphicEntry(0x143, GraphicIdentifier::compressed, 0x014, 0),
  // 0x46 (conveyor duck)
  MappingToGraphicEntry(0x144, GraphicIdentifier::compressed, 0x019, 0),
  MappingToGraphicEntry(0x145, GraphicIdentifier::compressed, 0x019, 0),
  // 0x47 (beehive)
  MappingToGraphicEntry(0x146, GraphicIdentifier::compressed, 0x020, 0),
  MappingToGraphicEntry(0x147, GraphicIdentifier::compressed, 0x020, 0),
  MappingToGraphicEntry(0x148, GraphicIdentifier::compressed, 0x020, 0),
  MappingToGraphicEntry(0x149, GraphicIdentifier::compressed, 0x020, 0),
  MappingToGraphicEntry(0x14A, GraphicIdentifier::compressed, 0x020, 0),
  // 0x48 (remote robot killer beams)
  MappingToGraphicEntry(0x14B, GraphicIdentifier::compressed, 0x017, 0),
  MappingToGraphicEntry(0x14C, GraphicIdentifier::compressed, 0x017, 0),
  MappingToGraphicEntry(0x14D, GraphicIdentifier::compressed, 0x017, 0),
  // 0x49 (back-and-forth laser)
  MappingToGraphicEntry(0x14E, GraphicIdentifier::compressed, 0x01B, 0),
  MappingToGraphicEntry(0x14F, GraphicIdentifier::compressed, 0x01B, 0),
  // 0x4A (Coco Island boss room door)
  MappingToGraphicEntry(0x150, GraphicIdentifier::compressed, 0x01C, 0),
  MappingToGraphicEntry(0x151, GraphicIdentifier::compressed, 0x01C, 0),
  // 0x4B (electric barrier)
  MappingToGraphicEntry(0x152, GraphicIdentifier::compressed, 0x01D, 0),
  MappingToGraphicEntry(0x153, GraphicIdentifier::compressed, 0x01D, 0),
  MappingToGraphicEntry(0x154, GraphicIdentifier::compressed, 0x01D, 0),
  MappingToGraphicEntry(0x155, GraphicIdentifier::compressed, 0x01D, 0),
  // 0x4C (electric barrier switch)
  MappingToGraphicEntry(0x156, GraphicIdentifier::compressed, 0x01E, 0),
  // 0x4D (sea fox weapon smoke puff)
  MappingToGraphicEntry(0x157, GraphicIdentifier::compressed, 0x023, 0),
  MappingToGraphicEntry(0x158, GraphicIdentifier::compressed, 0x023, 0),
  // 0x4E (pushable rock)
  MappingToGraphicEntry(0x159, GraphicIdentifier::compressed, 0x016, 0),
  MappingToGraphicEntry(0x15A, GraphicIdentifier::compressed, 0x016, 0),
  // 0x4F (wind gust effect)
  MappingToGraphicEntry(0x15B, GraphicIdentifier::compressed, 0x009, 0),
  MappingToGraphicEntry(0x15C, GraphicIdentifier::compressed, 0x009, 0),
  // 0x50 (world map: title card)
  MappingToGraphicEntry(0x15D, GraphicIdentifier::uncompressed, 0x104, 0),
  // i don't know what the fuck these are but they're not valid
  MappingToGraphicEntry(0x15E, GraphicIdentifier::uncompressed, 0x0D9, -0x0E),
  MappingToGraphicEntry(0x15F, GraphicIdentifier::uncompressed, 0x0D9, -0x16),
  MappingToGraphicEntry(0x160, GraphicIdentifier::uncompressed, 0x0D9, -0x24),
  MappingToGraphicEntry(0x161, GraphicIdentifier::uncompressed, 0x0D9, -0x32),
  MappingToGraphicEntry(0x162, GraphicIdentifier::uncompressed, 0x0D9, -0x32),
  MappingToGraphicEntry(0x163, GraphicIdentifier::uncompressed, 0x0D9, -0x32),
  MappingToGraphicEntry(0x164, GraphicIdentifier::uncompressed, 0x0D9, 0),
  MappingToGraphicEntry(0x165, GraphicIdentifier::uncompressed, 0x0D9, 0),
  MappingToGraphicEntry(0x166, GraphicIdentifier::uncompressed, 0x0D9, 0),
  MappingToGraphicEntry(0x167, GraphicIdentifier::uncompressed, 0x0D9, 0),
  MappingToGraphicEntry(0x168, GraphicIdentifier::uncompressed, 0x0D9, 0),
  MappingToGraphicEntry(0x169, GraphicIdentifier::uncompressed, 0x0D9, 0),
  // 0x51-0x5B (world map stuff. Every object is assigned all the mapping for
  // every other object for some reason)
  MappingToGraphicEntry(0x16A, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x16B, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x16C, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x16D, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x16E, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x16F, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x170, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x171, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x172, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x173, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x174, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x175, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x176, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x177, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x178, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x179, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x17A, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x17B, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x17C, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x17D, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x17E, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x17F, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x180, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x181, GraphicIdentifier::compressed, 0x022, 0),
  MappingToGraphicEntry(0x182, GraphicIdentifier::compressed, 0x022, 0),
  // 0x5C: log roller duck
  MappingToGraphicEntry(0x183, GraphicIdentifier::compressed, 0x026, 0),
  MappingToGraphicEntry(0x184, GraphicIdentifier::compressed, 0x026, 0),
  MappingToGraphicEntry(0x185, GraphicIdentifier::compressed, 0x026, 0),
  MappingToGraphicEntry(0x186, GraphicIdentifier::compressed, 0x026, 0),
  // 0x5D: spring
  MappingToGraphicEntry(0x187, GraphicIdentifier::compressed, 0x00A, 0),
  MappingToGraphicEntry(0x188, GraphicIdentifier::compressed, 0x00A, 0),
  // 0x5E: elevator
  MappingToGraphicEntry(0x189, GraphicIdentifier::compressed, 0x029, 0),
  // 0x5F-0x60: poloy forest boss
  MappingToGraphicEntry(0x18A, GraphicIdentifier::compressed, 0x02A, 0),
  MappingToGraphicEntry(0x18B, GraphicIdentifier::compressed, 0x02A, 0),
  MappingToGraphicEntry(0x18C, GraphicIdentifier::compressed, 0x02A, 0),
  MappingToGraphicEntry(0x18D, GraphicIdentifier::compressed, 0x02A, 0),
  MappingToGraphicEntry(0x18E, GraphicIdentifier::compressed, 0x02A, 0),
  MappingToGraphicEntry(0x18F, GraphicIdentifier::compressed, 0x02A, 0),
  MappingToGraphicEntry(0x190, GraphicIdentifier::compressed, 0x02A, 0),
  MappingToGraphicEntry(0x191, GraphicIdentifier::compressed, 0x02A, 0),
  MappingToGraphicEntry(0x192, GraphicIdentifier::compressed, 0x02A, 0),
  MappingToGraphicEntry(0x193, GraphicIdentifier::compressed, 0x02A, 0),
  MappingToGraphicEntry(0x194, GraphicIdentifier::compressed, 0x02A, 0),
  MappingToGraphicEntry(0x195, GraphicIdentifier::compressed, 0x02A, 0),
  MappingToGraphicEntry(0x196, GraphicIdentifier::compressed, 0x02A, 0),
  MappingToGraphicEntry(0x197, GraphicIdentifier::compressed, 0x02A, 0),
  MappingToGraphicEntry(0x198, GraphicIdentifier::compressed, 0x02A, 0),
  MappingToGraphicEntry(0x199, GraphicIdentifier::compressed, 0x02A, 0),
  // 0x61: cavern island boss
  MappingToGraphicEntry(0x19A, GraphicIdentifier::compressed, 0x033, 0),
  MappingToGraphicEntry(0x19B, GraphicIdentifier::compressed, 0x033, 0),
  MappingToGraphicEntry(0x19C, GraphicIdentifier::compressed, 0x033, 0),
  MappingToGraphicEntry(0x19D, GraphicIdentifier::compressed, 0x033, 0),
  MappingToGraphicEntry(0x19E, GraphicIdentifier::compressed, 0x033, 0),
  MappingToGraphicEntry(0x19F, GraphicIdentifier::compressed, 0x033, 0),
  MappingToGraphicEntry(0x1A0, GraphicIdentifier::compressed, 0x033, 0),
  MappingToGraphicEntry(0x1A1, GraphicIdentifier::compressed, 0x033, 0),
  MappingToGraphicEntry(0x1A2, GraphicIdentifier::compressed, 0x034, -0x36),
  MappingToGraphicEntry(0x1A3, GraphicIdentifier::compressed, 0x034, -0x36),
  MappingToGraphicEntry(0x1A4, GraphicIdentifier::compressed, 0x034, -0x36),
  MappingToGraphicEntry(0x1A5, GraphicIdentifier::compressed, 0x034, -0x36),
  MappingToGraphicEntry(0x1A6, GraphicIdentifier::compressed, 0x034, -0x36),
  MappingToGraphicEntry(0x1A7, GraphicIdentifier::compressed, 0x034, -0x36),
  MappingToGraphicEntry(0x1A8, GraphicIdentifier::compressed, 0x034, -0x36),
  // 0x62: volcanic tunnel boss
  MappingToGraphicEntry(0x1A9, GraphicIdentifier::compressed, 0x033, 0),
  MappingToGraphicEntry(0x1AA, GraphicIdentifier::compressed, 0x033, 0),
  MappingToGraphicEntry(0x1AB, GraphicIdentifier::compressed, 0x033, 0),
  MappingToGraphicEntry(0x1AC, GraphicIdentifier::compressed, 0x033, 0),
  MappingToGraphicEntry(0x1AD, GraphicIdentifier::compressed, 0x033, 0),
  MappingToGraphicEntry(0x1AE, GraphicIdentifier::compressed, 0x033, 0),
  MappingToGraphicEntry(0x1AF, GraphicIdentifier::compressed, 0x033, 0),
  MappingToGraphicEntry(0x1B0, GraphicIdentifier::compressed, 0x033, 0),
  MappingToGraphicEntry(0x1B1, GraphicIdentifier::compressed, 0x032, -0x36),
  MappingToGraphicEntry(0x1B2, GraphicIdentifier::compressed, 0x032, -0x36),
  MappingToGraphicEntry(0x1B3, GraphicIdentifier::compressed, 0x032, -0x36),
  MappingToGraphicEntry(0x1B4, GraphicIdentifier::compressed, 0x032, -0x36),
  MappingToGraphicEntry(0x1B5, GraphicIdentifier::compressed, 0x032, -0x36),
  // 0x63: lake rocky boss
  MappingToGraphicEntry(0x1B6, GraphicIdentifier::compressed, 0x035, 0),
  MappingToGraphicEntry(0x1B7, GraphicIdentifier::compressed, 0x035, 0),
  MappingToGraphicEntry(0x1B8, GraphicIdentifier::compressed, 0x035, 0),
  MappingToGraphicEntry(0x1B9, GraphicIdentifier::compressed, 0x035, 0),
  MappingToGraphicEntry(0x1BA, GraphicIdentifier::compressed, 0x035, 0),
  // 0x64: coco island boss
  MappingToGraphicEntry(0x1BB, GraphicIdentifier::compressed, 0x02B, 0),
  MappingToGraphicEntry(0x1BC, GraphicIdentifier::compressed, 0x02B, 0),
  MappingToGraphicEntry(0x1BD, GraphicIdentifier::compressed, 0x02B, 0),
  MappingToGraphicEntry(0x1BE, GraphicIdentifier::compressed, 0x02B, 0),
  MappingToGraphicEntry(0x1BF, GraphicIdentifier::compressed, 0x02B, 0),
  MappingToGraphicEntry(0x1C0, GraphicIdentifier::compressed, 0x02B, 0),
  MappingToGraphicEntry(0x1C1, GraphicIdentifier::compressed, 0x02B, 0),
  MappingToGraphicEntry(0x1C2, GraphicIdentifier::compressed, 0x02B, 0),
  MappingToGraphicEntry(0x1C3, GraphicIdentifier::compressed, 0x02B, 0),
  MappingToGraphicEntry(0x1C4, GraphicIdentifier::compressed, 0x02B, 0),
  // 0x65: polly mt 1 boss
  MappingToGraphicEntry(0x1C5, GraphicIdentifier::compressed, 0x02C, 0),
  MappingToGraphicEntry(0x1C6, GraphicIdentifier::compressed, 0x02C, 0),
  MappingToGraphicEntry(0x1C7, GraphicIdentifier::compressed, 0x02C, 0),
  MappingToGraphicEntry(0x1C8, GraphicIdentifier::compressed, 0x02C, 0),
  MappingToGraphicEntry(0x1C9, GraphicIdentifier::compressed, 0x02C, 0),
  MappingToGraphicEntry(0x1CA, GraphicIdentifier::compressed, 0x02C, 0),
  MappingToGraphicEntry(0x1CB, GraphicIdentifier::compressed, 0x02C, 0),
  MappingToGraphicEntry(0x1CC, GraphicIdentifier::compressed, 0x02C, 0),
  MappingToGraphicEntry(0x1CD, GraphicIdentifier::compressed, 0x02C, 0),
  MappingToGraphicEntry(0x1CE, GraphicIdentifier::compressed, 0x02C, 0),
  // 0x66: battle fortress boss 1
  MappingToGraphicEntry(0x1CF, GraphicIdentifier::compressed, 0x036, 0),
  MappingToGraphicEntry(0x1D0, GraphicIdentifier::compressed, 0x036, 0),
  MappingToGraphicEntry(0x1D1, GraphicIdentifier::compressed, 0x036, 0),
  MappingToGraphicEntry(0x1D2, GraphicIdentifier::compressed, 0x036, 0),
  MappingToGraphicEntry(0x1D3, GraphicIdentifier::compressed, 0x036, 0),
  MappingToGraphicEntry(0x1D4, GraphicIdentifier::compressed, 0x036, 0),
  MappingToGraphicEntry(0x1D5, GraphicIdentifier::compressed, 0x036, 0),
  MappingToGraphicEntry(0x1D6, GraphicIdentifier::compressed, 0x036, 0),
  MappingToGraphicEntry(0x1D7, GraphicIdentifier::compressed, 0x036, 0),
  MappingToGraphicEntry(0x1D8, GraphicIdentifier::compressed, 0x036, 0),
  MappingToGraphicEntry(0x1D9, GraphicIdentifier::compressed, 0x036, 0),
  MappingToGraphicEntry(0x1DA, GraphicIdentifier::compressed, 0x036, 0),
  MappingToGraphicEntry(0x1DB, GraphicIdentifier::compressed, 0x036, 0),
  MappingToGraphicEntry(0x1DC, GraphicIdentifier::compressed, 0x036, 0),
  // 0x67: battle fortress final boss
  MappingToGraphicEntry(0x1DD, GraphicIdentifier::uncompressed, 0x0D9, 0),
  MappingToGraphicEntry(0x1DE, GraphicIdentifier::uncompressed, 0x0D9, 0),
  MappingToGraphicEntry(0x1DF, GraphicIdentifier::uncompressed, 0x0D9, 0),
  MappingToGraphicEntry(0x1E0, GraphicIdentifier::uncompressed, 0x0D9, 0),
  MappingToGraphicEntry(0x1E1, GraphicIdentifier::uncompressed, 0x0D9, 0),
  MappingToGraphicEntry(0x1E2, GraphicIdentifier::uncompressed, 0x0D9, 0),
  MappingToGraphicEntry(0x1E3, GraphicIdentifier::uncompressed, 0x0D9, 0),
  MappingToGraphicEntry(0x1E4, GraphicIdentifier::uncompressed, 0x0D9, 0),
  MappingToGraphicEntry(0x1E5, GraphicIdentifier::uncompressed, 0x0D9, 0),
  MappingToGraphicEntry(0x1E6, GraphicIdentifier::uncompressed, 0x0D9, 0),
  MappingToGraphicEntry(0x1E7, GraphicIdentifier::uncompressed, 0x0D9, 0),
  MappingToGraphicEntry(0x1E8, GraphicIdentifier::uncompressed, 0x0D9, 0),
  MappingToGraphicEntry(0x1E9, GraphicIdentifier::uncompressed, 0x0D9, 0),
  // 0x68: conveyor bomb clone?
  // 0x69: flicky
  MappingToGraphicEntry(0x1EA, GraphicIdentifier::compressed, 0x02F, 0),
  MappingToGraphicEntry(0x1EB, GraphicIdentifier::compressed, 0x02F, 0),
  MappingToGraphicEntry(0x1EC, GraphicIdentifier::compressed, 0x02F, 0),
  MappingToGraphicEntry(0x1ED, GraphicIdentifier::compressed, 0x02F, 0),
  MappingToGraphicEntry(0x1EE, GraphicIdentifier::compressed, 0x030, -0x10),
  MappingToGraphicEntry(0x1EF, GraphicIdentifier::compressed, 0x030, -0x10),
  MappingToGraphicEntry(0x1F0, GraphicIdentifier::compressed, 0x030, -0x10),
  // 0x6A-0x6B: credits robot
  MappingToGraphicEntry(0x1F1, GraphicIdentifier::compressed, 0x031, 0),
  MappingToGraphicEntry(0x1F2, GraphicIdentifier::compressed, 0x031, 0),
  MappingToGraphicEntry(0x1F3, GraphicIdentifier::compressed, 0x031, 0),
  MappingToGraphicEntry(0x1F4, GraphicIdentifier::compressed, 0x031, 0),
  MappingToGraphicEntry(0x1F5, GraphicIdentifier::compressed, 0x031, 0),
  // null entries
  MappingToGraphicEntry(0x1F6, GraphicIdentifier::uncompressed, 0x0, 0),
  MappingToGraphicEntry(0x1F7, GraphicIdentifier::uncompressed, 0x0, 0),
  MappingToGraphicEntry(0x1F8, GraphicIdentifier::uncompressed, 0x0, 0),
  
  };


};
