#include "gamedata/TileMapInfos.h"
#include "exception/OutOfRangeIndexException.h"

namespace Tales {


int TileMapInfos::numTileMaps() {
  return numTileMaps_;
}

TileMapInfo TileMapInfos::tileMapInfo(int index) {
  if (index >= numTileMaps_) {
    throw OutOfRangeIndexException(TALES_SRCANDLINE,
                                   "TileMapInfos::tileMapInfo(int)",
                                   index);
  }

  return tileMapInfos_[index];
}

const TileMapInfo TileMapInfos::tileMapInfos_[numTileMaps_] =
  {
    TileMapInfo(0x403A0, 0x14, 0x12, TileMap::twoBytesPerTile,
                63, -0xC0, 1, "Sega logo"),
    TileMapInfo(0x4E41C, 0x14, 0x13, TileMap::twoBytesPerTile,
                68, -0x100, 28, "World map: Background"),
    TileMapInfo(0x4E714, 0x08, 0x03, TileMap::twoBytesPerTile,
                68, -0x100, 28, "World map: EN title box"),
    TileMapInfo(0x4D4C6, 0x14, 0x12, TileMap::oneBytePerTile,
                67, -0x100, 26, "Title screen: Background", 0x01),
    TileMapInfo(0x4D4BE, 0x08, 0x01, TileMap::oneBytePerTile,
                67, -0x100, 26, "Title screen: Sega copyright", 0x01),
    TileMapInfo(0x4D4B2, 0x0C, 0x01, TileMap::oneBytePerTile,
                67, -0x100, 26, "Title screen: PRESS START", 0x01),
    TileMapInfo(0x4D4C6, 0x0C, 0x01, TileMap::oneBytePerTile,
                67, -0x100, 26, "Title screen: PRESS START (blinking)", 0x01),
    TileMapInfo(0x4D4AA, 0x08, 0x01, TileMap::oneBytePerTile,
                67, -0x100, 26, "Title screen: NEW GAME", 0x01),
    TileMapInfo(0x4D4A4, 0x06, 0x01, TileMap::oneBytePerTile,
                67, -0x100, 26, "Title screen: CONTINUE", 0x01),
//    TileMapInfo(0x5AFA6, 0x14, 0x13, TileMap::twoBytesPerTile,
//                73, -0x100, 35, "Password: Background"),
    TileMapInfo(0x5AFA6, 0x14, 0x12, TileMap::twoBytesPerTile,
                73, -0x100, 35, "Password: Background"),
    TileMapInfo(0x5B276, 0xC, 0x2, TileMap::twoBytesPerTile,
                73, -0x100, 35, "Password: Main menu options JP dupe?"),
    TileMapInfo(0x5B2A6, 0xC, 0x2, TileMap::twoBytesPerTile,
                73, -0x100, 35, "Password: Dock options JP dupe?"),
    TileMapInfo(0x5B2D6, 0xC, 0x2, TileMap::twoBytesPerTile,
                73, -0x100, 35, "Password: Item menu options JP dupe?"),
    TileMapInfo(0x5B306, 0xC, 0x2, TileMap::twoBytesPerTile,
                73, -0x100, 35,
                "Password: Dock item menu options JP [unused]"),
//    TileMapInfo(0x5B89A, 0x2, 0x10, TileMap::twoBytesPerTile,
//                66, -0x100, 10, "???"),
//    TileMapInfo(0x5BB5D, 0xC, 0x4, TileMap::twoBytesPerTile,
//                80, -0xA2, 35, "???"),
//    TileMapInfo(0x5BCC0, 0xC, 0x2, TileMap::twoBytesPerTile,
//                80, -0x166, 35, "Password: Main menu options EN dupe?"),
    TileMapInfo(0x2CC48, 0x02, 0x02, TileMap::twoBytesPerTile,
                73, -0x100, 35, "Password: 0"),
    TileMapInfo(0x2CC50, 0x02, 0x02, TileMap::twoBytesPerTile,
                73, -0x100, 35, "Password: 1"),
    TileMapInfo(0x2CC58, 0x02, 0x02, TileMap::twoBytesPerTile,
                73, -0x100, 35, "Password: 2"),
    TileMapInfo(0x2CC60, 0x02, 0x02, TileMap::twoBytesPerTile,
                73, -0x100, 35, "Password: 3"),
    TileMapInfo(0x2CC68, 0x02, 0x02, TileMap::twoBytesPerTile,
                73, -0x100, 35, "Password: 4"),
    TileMapInfo(0x2CC70, 0x02, 0x02, TileMap::twoBytesPerTile,
                73, -0x100, 35, "Password: 5"),
    TileMapInfo(0x2CC78, 0x02, 0x02, TileMap::twoBytesPerTile,
                73, -0x100, 35, "Password: 6"),
    TileMapInfo(0x2CC80, 0x02, 0x02, TileMap::twoBytesPerTile,
                73, -0x100, 35, "Password: 7"),
    TileMapInfo(0x2CC88, 0x02, 0x02, TileMap::twoBytesPerTile,
                73, -0x100, 35, "Password: 8"),
    TileMapInfo(0x2CC90, 0x02, 0x02, TileMap::twoBytesPerTile,
                73, -0x100, 35, "Password: 9"),
    TileMapInfo(0x2CC98, 0x02, 0x02, TileMap::twoBytesPerTile,
                73, -0x100, 35, "Password: A"),
    TileMapInfo(0x2CCA0, 0x02, 0x02, TileMap::twoBytesPerTile,
                73, -0x100, 35, "Password: B"),
    TileMapInfo(0x2CCA8, 0x02, 0x02, TileMap::twoBytesPerTile,
                73, -0x100, 35, "Password: C"),
    TileMapInfo(0x2CCB0, 0x02, 0x02, TileMap::twoBytesPerTile,
                73, -0x100, 35, "Password: D"),
    TileMapInfo(0x2CCB8, 0x02, 0x02, TileMap::twoBytesPerTile,
                73, -0x100, 35, "Password: E"),
    TileMapInfo(0x2CCC0, 0x02, 0x02, TileMap::twoBytesPerTile,
                73, -0x100, 35, "Password: F"),
    // game actually uses 14x13, but that spills over into the
    // dock background
    TileMapInfo(0x2E066, 0x14, 0x12, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Tails' House: Interface"),
//    TileMapInfo(0x2E336, 0x12, 0x7, TileMap::twoBytesPerTile,
//                69, -0xA2, 35, "Tails' House: ?"),
    TileMapInfo(0x2EBC6, 0x12, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Tails' House: Blank between screens"),
    TileMapInfo(0x2EBA2, 0x12, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Tails' House: Shutter between screens"),
    TileMapInfo(0x2EA63, 0x1, 0x2, TileMap::twoBytesPerTile,
                69, -0xA2, 35,
                "Tails' House: Buttons L (top-left selected)"),
    TileMapInfo(0x2EA67, 0x1, 0x2, TileMap::twoBytesPerTile,
                69, -0xA2, 35,
                "Tails' House: Buttons R (top-left selected)"),
    TileMapInfo(0x2EA6C, 0x1, 0x2, TileMap::twoBytesPerTile,
                69, -0xA2, 35,
                "Tails' House: Buttons L (top-right selected)"),
    TileMapInfo(0x2EA70, 0x1, 0x2, TileMap::twoBytesPerTile,
                69, -0xA2, 35,
                "Tails' House: Buttons R (top-right selected)"),
    TileMapInfo(0x2EA75, 0x1, 0x2, TileMap::twoBytesPerTile,
                69, -0xA2, 35,
                "Tails' House: Buttons L (lower-left selected)"),
    TileMapInfo(0x2EA79, 0x1, 0x2, TileMap::twoBytesPerTile,
                69, -0xA2, 35,
                "Tails' House: Buttons R (lower-left selected)"),
    TileMapInfo(0x2EA7E, 0x1, 0x2, TileMap::twoBytesPerTile,
                69, -0xA2, 35,
                "Tails' House: Buttons L (lower-right selected)"),
    TileMapInfo(0x2EA82, 0x1, 0x2, TileMap::twoBytesPerTile,
                69, -0xA2, 35,
                "Tails' House: Buttons R (lower-right selected)"),
    TileMapInfo(0x2E432, 0x12, 0x7, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Tails' House: Main menu background"),
    TileMapInfo(0x2F1E6, 0xC, 0x2, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Tails' House: Main menu options EN"),
    TileMapInfo(0x2E9D2, 0xC, 0x2, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Tails' House: Main menu options JP"),
    TileMapInfo(0x2E336, 0x12, 0x7, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Tails' House: Dock background"),
    TileMapInfo(0x2F216, 0xC, 0x2, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Tails' House: Dock options EN"),
    TileMapInfo(0x2EA02, 0xC, 0x2, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Tails' House: Dock options JP"),
    TileMapInfo(0x2EA86, 0x12, 0x7, TileMap::twoBytesPerTile,
                57, 0, 4, "Tails' House: Item menu background"),
    TileMapInfo(0x2F246, 0xC, 0x2, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Tails' House: Item menu options EN"),
    TileMapInfo(0x2EA32, 0xC, 0x2, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Tails' House: Item menu options JP"),
    TileMapInfo(0x2EB9A, 0x1, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu: Left arrow disabled"),
    TileMapInfo(0x2EB9E, 0x1, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu: Right arrow disabled"),
    TileMapInfo(0x2EB82, 0x1, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu: Subscreen 1, left arrow enabled"),
    TileMapInfo(0x2EB86, 0x1, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu: Subscreen 1, right arrow enabled"),
    TileMapInfo(0x2EB8A, 0x1, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu: Subscreen 2, left arrow enabled"),
    TileMapInfo(0x2EB8E, 0x1, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu: Subscreen 2, right arrow enabled"),
    TileMapInfo(0x2EB92, 0x1, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu: Subscreen 3, left arrow enabled"),
    TileMapInfo(0x2EB96, 0x1, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu: Subscreen 3, right arrow enabled"),
    TileMapInfo(0x2DF66, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Regular Bomb"),
    TileMapInfo(0x2DF6E, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Large Bomb"),
    TileMapInfo(0x2DF76, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Remote Bomb"),
    TileMapInfo(0x2DF7E, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Napalm Bomb"),
    TileMapInfo(0x2DF86, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Triple Bomb"),
    TileMapInfo(0x2DF8E, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Wrench"),
    TileMapInfo(0x2DF96, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Helmet"),
    TileMapInfo(0x2DF5E, 0x2, 0x2, TileMap::twoBytesPerTile,
                58, -0x9E, 4, "Item menu icon: Remote Robot"),
    TileMapInfo(0x2DFAE, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Hammer"),
    TileMapInfo(0x2DFB6, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Teleport Device"),
    TileMapInfo(0x2DFBE, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Night Vision"),
    TileMapInfo(0x2DFC6, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Speed Boots"),
    TileMapInfo(0x2DFCE, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Super Glove"),
    TileMapInfo(0x2DFD6, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Fang"),
    TileMapInfo(0x2DFDE, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Knuckles"),
    TileMapInfo(0x2DFE6, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Sonic"),
    TileMapInfo(0x2DF9E, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Item Radar"),
    TileMapInfo(0x2DFA6, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Radio"),
    TileMapInfo(0x2E02E, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Blue Chaos Emerald"),
    TileMapInfo(0x2E036, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Green Chaos Emerald"),
    TileMapInfo(0x2E03E, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Purple Chaos Emerald"),
    TileMapInfo(0x2E046, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Red Chaos Emerald"),
    TileMapInfo(0x2E04E, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: White Chaos Emerald"),
    TileMapInfo(0x2E056, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Yellow Chaos Emerald"),
    TileMapInfo(0x2DFEE, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Proton Torpedo"),
    TileMapInfo(0x2DFF6, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Vulcan Gun"),
    TileMapInfo(0x2DFFE, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Extra Speed"),
    TileMapInfo(0x2E006, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Extra Armor"),
    TileMapInfo(0x2E00E, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Anti-Air Missile"),
    TileMapInfo(0x2E016, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Spark"),
    TileMapInfo(0x2E01E, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Mine"),
    TileMapInfo(0x2E026, 0x2, 0x2, TileMap::twoBytesPerTile,
                57, 0, 4, "Item menu icon: Rocket Booster"),
    TileMapInfo(0x2ED80, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: REGULAR BOMB"),
    TileMapInfo(0x2EDAE, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: LARGE BOMB"),
    TileMapInfo(0x2EDD4, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: REMOTE BOMB"),
    TileMapInfo(0x2EE26, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: NAPALM BOMB"),
    TileMapInfo(0x2EDFC, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: TRIPLE BOMB"),
    TileMapInfo(0x2EE78, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: WRENCH"),
    TileMapInfo(0x2EEA0, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: HELMET"),
    TileMapInfo(0x2EE50, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: REMOTE ROBOT"),
    TileMapInfo(0x2EF10, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: HAMMER"),
    TileMapInfo(0x2EF38, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: TELEPORT DEVICE"),
    TileMapInfo(0x2EF6A, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: NIGHT VISION"),
    TileMapInfo(0x2EF92, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: SPEED BOOTS"),
    TileMapInfo(0x2EFB8, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: SUPER GLOVE"),
    TileMapInfo(0x2EFDE, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: FANG"),
    TileMapInfo(0x2F00E, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: KNUCKLES"),
    TileMapInfo(0x2F03E, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: SONIC"),
    TileMapInfo(0x2EEC8, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: ITEM RADAR"),
    TileMapInfo(0x2EEEC, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: RAIDO"),
    TileMapInfo(0x2ECF4, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: BLUE C.EMERALD"),
    TileMapInfo(0x2EC6A, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: GREEN C.EMERALD"),
    TileMapInfo(0x2ED20, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: PURPLE C.EMERALD"),
    TileMapInfo(0x2ECCA, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: RED C.EMERALD"),
    TileMapInfo(0x2EC9A, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: WHITE C.EMERALD"),
    TileMapInfo(0x2ED50, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: YELLOW C.EMERALD"),
    TileMapInfo(0x2F062, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: PROTON TORPEDO"),
    TileMapInfo(0x2F08E, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: VULCAN GUN"),
    TileMapInfo(0x2F0BC, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: EXTRA SPEED"),
    TileMapInfo(0x2F0F2, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: EXTRA ARMOR"),
    TileMapInfo(0x2F128, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: ANTI-AIRMISSILE"),
    TileMapInfo(0x2F158, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: SPARK"),
    TileMapInfo(0x2F18C, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: MINE"),
    TileMapInfo(0x2F1BC, 0x11, 0x1, TileMap::twoBytesPerTile,
                80, -0x166, 35, "Item menu label: ROCKET BOOTER"),
    TileMapInfo(0x2E6BA, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Regular Bomb (JP)"),
    TileMapInfo(0x2E6DE, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Large Bomb (JP)"),
    TileMapInfo(0x2E88E, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Remote Bomb (JP)"),
    TileMapInfo(0x2E822, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Napalm Bomb (JP)"),
    TileMapInfo(0x2E64E, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Triple Bomb (JP)"),
    TileMapInfo(0x2E91E, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Wrench (JP)"),
    TileMapInfo(0x2E726, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Helmet (JP)"),
    TileMapInfo(0x2E9AE, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Remote Robot (JP)"),
    TileMapInfo(0x2E702, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Hammer (JP)"),
    TileMapInfo(0x2E966, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Teleport Device (JP)"),
    TileMapInfo(0x2E8D6, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Night Vision (JP)"),
    TileMapInfo(0x2E552, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Speed Boots (JP)"),
    TileMapInfo(0x2E942, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Super Glove (JP)"),
    TileMapInfo(0x2E696, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Fang (JP)"),
    TileMapInfo(0x2E7B6, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Knuckles (JP)"),
    TileMapInfo(0x2E8FA, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Sonic (JP)"),
    TileMapInfo(0x2E74A, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Item Radar (JP)"),
    TileMapInfo(0x2E86A, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Radio (JP)"),
    TileMapInfo(0x2E59A, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Blue Chaos Emerald (JP)"),
    TileMapInfo(0x2E5BE, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Green Chaos Emerald (JP)"),
    TileMapInfo(0x2E62A, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Purple Chaos Emerald (JP)"),
    TileMapInfo(0x2E576, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Red Chaos Emerald (JP)"),
    TileMapInfo(0x2E5E2, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: White Chaos Emerald (JP)"),
    TileMapInfo(0x2E606, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Yellow Chaos Emerald (JP)"),
    TileMapInfo(0x2E846, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Proton Torpedo (JP)"),
    TileMapInfo(0x2E7FE, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Vulcan Gun (JP)"),
    TileMapInfo(0x2E76E, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Extra Speed (JP)"),
    TileMapInfo(0x2E7DA, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Extra Armor (JP)"),
    TileMapInfo(0x2E98A, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Anti-Air Missile (JP)"),
    TileMapInfo(0x2E672, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Spark (JP)"),
    TileMapInfo(0x2E792, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Mine (JP)"),
    TileMapInfo(0x2E8B2, 0x11, 0x1, TileMap::twoBytesPerTile,
                69, -0xA2, 35, "Item menu label: Rocket Booster (JP)"),
    TileMapInfo(0x5B5FA, 0x14, 0x2, TileMap::twoBytesPerTile,
                75, -0x20, 38, "Game over: GAME OVER"),
    TileMapInfo(0x5B64A, 0x8, 0x2, TileMap::twoBytesPerTile,
                75, -0x20, 38, "Game over: Continue (JP) [unused]"),
    TileMapInfo(0x5B64A + 0x20, 0xA, 0x2, TileMap::twoBytesPerTile,
                75, -0x20, 38, "Game over: Quit (JP) [unused]"),
    TileMapInfo(0x5B692, 0x1, 0x2, TileMap::twoBytesPerTile,
                75, -0x20, 38, "Game over: Cursor 1 [unused]"),
    TileMapInfo(0x5B696, 0x1, 0x2, TileMap::twoBytesPerTile,
                75, -0x20, 38, "Game over: Cursor 2 [unused]"),
    TileMapInfo(0x62D74, 0x14, 0x13, TileMap::twoBytesPerTile,
                78, -0x100, 37, "Credits: Background", 0x01),
    TileMapInfo(0x3A253, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 1", 0x01),
    TileMapInfo(0x3A290, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 2", 0x01),
    TileMapInfo(0x3A2CD, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 3", 0x01),
    TileMapInfo(0x3A30A, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 4", 0x01),
    TileMapInfo(0x3A347, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 5", 0x01),
    TileMapInfo(0x3A384, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 6", 0x01),
    TileMapInfo(0x3A3C1, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 7", 0x01),
    TileMapInfo(0x3A3FE, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 8", 0x01),
    TileMapInfo(0x3A43B, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 9", 0x01),
    TileMapInfo(0x3A478, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 10", 0x01),
    TileMapInfo(0x3A4B5, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 11", 0x01),
    TileMapInfo(0x3A4F2, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 12", 0x01),
    TileMapInfo(0x3A52F, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 13", 0x01),
    TileMapInfo(0x3A56C, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 14", 0x01),
    TileMapInfo(0x3A5A9, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 15", 0x01),
    TileMapInfo(0x3A5E6, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 16", 0x01),
    TileMapInfo(0x3A623, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 17", 0x01),
    TileMapInfo(0x3A660, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 18", 0x01),
    TileMapInfo(0x3A69D, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 19", 0x01),
    TileMapInfo(0x3A6DA, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 20", 0x01),
    TileMapInfo(0x3A717, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 21", 0x01),
    TileMapInfo(0x3A754, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 22", 0x01),
    TileMapInfo(0x3A791, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 23", 0x01),
    TileMapInfo(0x3A7CE, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 24", 0x01),
    TileMapInfo(0x3A80B, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 25", 0x01),
    TileMapInfo(0x3A848, 0x14, 0x3, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Section 26", 0x01),
    TileMapInfo(0x3A885, 0x14, 0x6, TileMap::oneBytePerTile,
                78, -0x100, 37, "Credits: Item completion message", 0x01),
    TileMapInfo(0x43BF2, 0x13, 0x11, TileMap::oneBytePerTile,
                64, -0x120, 2, "Level select [unused]", 0x01),
    TileMapInfo(0x43D6D, 0x13, 0xA, TileMap::oneBytePerTile,
                64, -0x120, 2, "Flight debug menu [unused]", 0x01),
  };


};
